@@ -42,14 +42,15 @@ void afs_PrefetchChunk(struct vcache *avc, struct dcache *adc,
 		       afs_ucred_t *acred, struct vrequest *areq);
 
 int
-afs_MemRead(register struct vcache *avc, struct uio *auio,
+afs_MemRead(struct vcache *avc, struct uio *auio,
 	    afs_ucred_t *acred, daddr_t albn, struct buf **abpp,
 	    int noLock)
 {
     afs_size_t totalLength;
     afs_size_t transferLength;
     afs_size_t filePos;
-    afs_size_t offset, len, tlen;
+    afs_size_t offset, tlen;
+    afs_size_t len = 0;
     afs_int32 trimlen;
     struct dcache *tdc = 0;
     afs_int32 error, trybusy = 1;
@@ -110,6 +111,16 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
      * Locks held:
      * avc->lock(R)
      */
+
+    /* This bit is bogus. We're checking to see if the read goes past the
+     * end of the file. If so, we should be zeroing out all of the buffers
+     * that the client has passed into us (there is a danger that we may leak
+     * kernel memory if we do not). However, this behaviour is disabled by
+     * not setting len before this segment runs, and by setting len to 0
+     * immediately we enter it. In addition, we also need to check for a read
+     * which partially goes off the end of the file in the while loop below.
+     */
+
     if (filePos >= avc->f.m.Length) {
 	if (len > AFS_ZEROS)
 	    len = sizeof(afs_zeros);	/* and in 0 buffer */
@@ -196,7 +207,7 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
 			tdc->mflags |= DFFetchReq;
 			bp = afs_BQueue(BOP_FETCH, avc, B_DONTWAIT, 0, acred,
 					(afs_size_t) filePos, (afs_size_t) 0,
-					tdc);
+					tdc, (void *)0, (void *)0);
 			if (!bp) {
 			    tdc->mflags &= ~DFFetchReq;
 			    trybusy = 0;	/* Avoid bkg daemon since they're too busy */
@@ -418,7 +429,7 @@ void
 afs_PrefetchChunk(struct vcache *avc, struct dcache *adc,
 		  afs_ucred_t *acred, struct vrequest *areq)
 {
-    register struct dcache *tdc;
+    struct dcache *tdc;
     afs_size_t offset;
     afs_size_t j1, j2;		/* junk vbls for GetDCache to trash */
 
@@ -436,14 +447,12 @@ afs_PrefetchChunk(struct vcache *avc, struct dcache *adc,
 	ReleaseReadLock(&adc->lock);
 
 	tdc = afs_GetDCache(avc, offset, areq, &j1, &j2, 2);	/* type 2 never returns 0 */
-#ifdef AFS_DISCON_ENV
         /*
          * In disconnected mode, type 2 can return 0 because it doesn't
          * make any sense to allocate a dcache we can never fill
          */
          if (tdc == NULL)
              return;
-#endif /* AFS_DISCON_ENV */
 
 	ObtainSharedLock(&tdc->mflock, 651);
 	if (!(tdc->mflags & DFFetchReq)) {
@@ -454,7 +463,8 @@ afs_PrefetchChunk(struct vcache *avc, struct dcache *adc,
 	     * since we don't want to wait for it to finish before doing so ourselves.
 	     */
 	    bp = afs_BQueue(BOP_FETCH, avc, B_DONTWAIT, 0, acred,
-			    (afs_size_t) offset, (afs_size_t) 1, tdc);
+			    (afs_size_t) offset, (afs_size_t) 1, tdc,
+			    (void *)0, (void *)0);
 	    if (!bp) {
 		/* Bkg table full; just abort non-important prefetching to avoid deadlocks */
 		tdc->mflags &= ~DFFetchReq;
@@ -487,14 +497,15 @@ afs_PrefetchChunk(struct vcache *avc, struct dcache *adc,
 }
 
 int
-afs_UFSRead(register struct vcache *avc, struct uio *auio,
+afs_UFSRead(struct vcache *avc, struct uio *auio,
 	    afs_ucred_t *acred, daddr_t albn, struct buf **abpp,
 	    int noLock)
 {
     afs_size_t totalLength;
     afs_size_t transferLength;
     afs_size_t filePos;
-    afs_size_t offset, len, tlen;
+    afs_size_t offset, tlen;
+    afs_size_t len = 0;
     afs_int32 trimlen;
     struct dcache *tdc = 0;
     afs_int32 error;
@@ -561,6 +572,15 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
     }
 #endif
 
+    /* This bit is bogus. We're checking to see if the read goes past the
+     * end of the file. If so, we should be zeroing out all of the buffers
+     * that the client has passed into us (there is a danger that we may leak
+     * kernel memory if we do not). However, this behaviour is disabled by
+     * not setting len before this segment runs, and by setting len to 0
+     * immediately we enter it. In addition, we also need to check for a read
+     * which partially goes off the end of the file in the while loop below.
+     */
+
     if (filePos >= avc->f.m.Length) {
 	if (len > AFS_ZEROS)
 	    len = sizeof(afs_zeros);	/* and in 0 buffer */
@@ -627,13 +647,10 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 		afs_PutDCache(tdc);	/* before reusing tdc */
 	    }
 	    tdc = afs_GetDCache(avc, filePos, &treq, &offset, &len, 2);
-#ifdef AFS_DISCON_ENV
 	    if (!tdc) {
-		printf("Network down in afs_read");
 	        error = ENETDOWN;
 	        break;
 	    }
-#endif /* AFS_DISCON_ENV */
 
 	    ObtainReadLock(&tdc->lock);
 	    /* now, first try to start transfer, if we'll need the data.  If
@@ -652,7 +669,7 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 			tdc->mflags |= DFFetchReq;
 			bp = afs_BQueue(BOP_FETCH, avc, B_DONTWAIT, 0, acred,
 					(afs_size_t) filePos, (afs_size_t) 0,
-					tdc);
+					tdc, (void *)0, (void *)0);
 			if (!bp) {
 			    /* Bkg table full; retry deadlocks */
 			    tdc->mflags &= ~DFFetchReq;
@@ -882,12 +899,19 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 	    code = VOP_READ(tfile->vnode, &tuio, 0, afs_osi_credp);
 	    VOP_UNLOCK(tfile->vnode, 0);
 	    AFS_GLOCK();
-#elif defined(AFS_FBSD50_ENV)
+#elif defined(AFS_FBSD_ENV)
 	    AFS_GUNLOCK();
 	    VOP_LOCK(tfile->vnode, LK_EXCLUSIVE, curthread);
 	    code = VOP_READ(tfile->vnode, &tuio, 0, afs_osi_credp);
 	    VOP_UNLOCK(tfile->vnode, 0, curthread);
 	    AFS_GLOCK();
+#elif defined(AFS_NBSD_ENV)
+	    AFS_GUNLOCK();
+	    VOP_LOCK(tfile->vnode, LK_EXCLUSIVE);
+	    code = VOP_READ(tfile->vnode, &tuio, 0, afs_osi_credp);
+	    VOP_UNLOCK(tfile->vnode, 0);
+	    AFS_GLOCK();
+
 #elif defined(AFS_XBSD_ENV)
 	    AFS_GUNLOCK();
 	    VOP_LOCK(tfile->vnode, LK_EXCLUSIVE, curproc);
